@@ -112,9 +112,24 @@ def create_resource_mapping(context, top_id, bottom_id, pod_id, project_id,
         context.session.close()
 
 
-def list_resource_routings(context, filters=None, sorts=None):
+def list_resource_routings(context, filters=None, limit=None, marker=None,
+                           sorts=None):
+    """Return a list of limited number of resource routings
+
+    :param context:
+    :param filters: list of filter dict with key 'key', 'comparator', 'value'
+    :param limit: an integer that limits the maximum number of items
+                  returned in a single response
+    :param marker: id of the last item in the previous list
+    :param sorts: a list of (sort_key, sort_dir) pair,
+                  for example, [('id', 'desc')]
+    :return: a list of limited number of items
+    """
     with context.session.begin():
-        return core.query_resource(context, models.ResourceRouting,
+        return core.paginate_query(context, models.ResourceRouting,
+                                   limit,
+                                   models.ResourceRouting(
+                                       id=marker) if marker else None,
                                    filters or [], sorts or [])
 
 
@@ -413,33 +428,6 @@ def get_latest_failed_or_new_jobs(context):
     return failed_jobs, new_jobs
 
 
-def list_jobs(context, filters=None, sorts=None):
-    with context.session.begin():
-        # get all jobs from job table
-        jobs = core.query_resource(context, models.AsyncJob,
-                                   filters or [], sorts or [])
-        return jobs
-
-
-def list_jobs_from_log(context, filters=None, sorts=None):
-    with context.session.begin():
-        # get all jobs from job log table, because the job log table only
-        # stores successful jobs, so this method merely returns successful jobs
-        if filters is not None:
-            for filter in filters:
-                if filter.get('key') == 'status':
-                    job_status = filter['value']
-                    # job entry in job log table has no status attribute.
-                    if job_status == constants.JS_Success:
-                        filters.remove(filter)
-                    else:
-                        return []
-
-        jobs_in_log = core.query_resource(
-            context, models.AsyncJobLog, filters or [], sorts or [])
-        return jobs_in_log
-
-
 def get_job(context, job_id):
     with context.session.begin():
         return core.get_resource(context, models.AsyncJob, job_id)
@@ -455,6 +443,44 @@ def delete_job(context, job_id):
         return core.delete_resource(context, models.AsyncJob, job_id)
 
 
+def list_jobs(context, filters=None, sorts=None, limit=None, marker=None):
+    with context.session.begin():
+        marker_obj = None
+        if marker is not None:
+            marker_obj = context.session.query(models.AsyncJob).filter(
+                models.AsyncJob.id == marker).first()
+        return core.paginate_query(
+            context, models.AsyncJob, limit, marker_obj,
+            filters or [], sorts or [])
+
+
+def list_jobs_from_log(context, filters=None, sorts=None,
+                       limit=None, marker=None):
+    with context.session.begin():
+        marker_obj = None
+        if marker is not None:
+            marker_obj = context.session.query(models.AsyncJobLog).filter(
+                models.AsyncJobLog.id == marker).first()
+
+        filter_is_success = True
+        if filters is not None and len(filters) > 0:
+            for filter in filters:
+                if filter.get('key') == 'status':
+                    job_status = filter['value']
+                    # job entry in job log table has no
+                    # status attribute.
+                    if job_status == constants.JS_Success:
+                        filters.remove(filter)
+                    else:
+                        filter_is_success = False
+                    break
+        if filter_is_success:
+            return core.paginate_query(context, models.AsyncJobLog, limit,
+                                       marker_obj,
+                                       filters or [], sorts or [])
+        return []
+
+
 def get_latest_job(context, status, _type, resource_id):
     jobs = core.query_resource(
         context, models.AsyncJob,
@@ -678,3 +704,33 @@ def is_valid_model_filters(model, filters):
         if not hasattr(model, key):
             return False
     return True
+
+
+def create_recycle_resource(context, resource_id, resource_type, project_id):
+    try:
+        context.session.begin()
+        route = core.create_resource(context, models.RecycleResources,
+                                     {'resource_id': resource_id,
+                                      'resource_type': resource_type,
+                                      'project_id': project_id})
+        context.session.commit()
+        return route
+    except db_exc.DBDuplicateEntry:
+        # entry has already been created
+        context.session.rollback()
+        return None
+    finally:
+        context.session.close()
+
+
+def list_recycle_resources(context, filters=None, sorts=None):
+    with context.session.begin():
+        resources = core.query_resource(
+            context, models.RecycleResources, filters or [], sorts or [])
+        return resources
+
+
+def delete_recycle_resource(context, resource_id):
+    with context.session.begin():
+        return core.delete_resource(
+            context, models.RecycleResources, resource_id)
