@@ -23,7 +23,6 @@ from oslo_db.sqlalchemy import utils as sa_utils
 import oslo_log.helpers as log_helpers
 from oslo_log import log
 
-from neutron.api.v2 import attributes
 from neutron.callbacks import events
 from neutron.callbacks import exceptions as callbacks_exc
 from neutron.callbacks import registry
@@ -51,6 +50,7 @@ from neutron.extensions import providernet as provider
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net
 from neutron_lib.api import validators
+import neutron_lib.callbacks.resources as attributes
 from neutron_lib import constants
 from neutron_lib import exceptions
 from neutron_lib.plugins import directory
@@ -75,11 +75,11 @@ from tricircle.network import security_groups
 
 tricircle_opts = [
     cfg.ListOpt('type_drivers',
-                default=['local,vxlan'],
+                default=['vxlan,local'],
                 help=_('List of network type driver entry points to be loaded '
                        'from the tricircle.network.type_drivers namespace.')),
     cfg.ListOpt('tenant_network_types',
-                default=['local,vxlan'],
+                default=['vxlan,local'],
                 help=_('Ordered list of network_types to allocate as tenant '
                        'networks. The default value "local" is useful for '
                        'single pod connectivity.')),
@@ -1733,10 +1733,35 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         return return_info
 
     def create_floatingip(self, context, floatingip):
-        # create bottom fip when associating fixed ip
-        return super(TricirclePlugin, self).create_floatingip(
-            context, floatingip,
-            initial_status=constants.FLOATINGIP_STATUS_DOWN)
+        fip = None
+        t_ctx = t_context.get_context_from_neutron_context(context)
+        try:
+            fip = super(TricirclePlugin, self).create_floatingip(
+                context, floatingip,
+                initial_status=constants.FLOATINGIP_STATUS_DOWN)
+            if fip['router_id'] and fip['port_id']:
+                mappings = db_api.get_bottom_mappings_by_top_id(
+                    t_ctx, fip['port_id'], t_constants.RT_PORT)
+                if not mappings:
+                    # mapping does not exist, meaning that the bottom port has
+                    # not been created, we just return and defer the work to
+                    # setup bottom floating ip until vm creation
+                    return fip
+
+                int_net_pod, b_int_port_id = mappings[0]
+                int_port = self.get_port(context, fip['port_id'])
+                net_id = int_port['network_id']
+                router = self._get_router(context, fip['router_id'])
+                self.xjob_handler.setup_bottom_router(
+                    t_ctx, router['tenant_id'], net_id,
+                    fip['router_id'], int_net_pod['pod_id'])
+            return fip
+        except Exception:
+            if fip:
+                # if we fail to register the job, delete the fip
+                super(TricirclePlugin, self).delete_floatingip(context,
+                                                               fip['id'])
+            raise
 
     def remove_router_interface(self, context, router_id, interface_info):
         t_ctx = t_context.get_context_from_neutron_context(context)
@@ -1875,8 +1900,8 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             t_ctx, int_port_id, t_constants.RT_PORT)
         if not mappings:
             # mapping does not exist, meaning that the bottom port has not
-            # been created, we just return and leave the work to setup bottom
-            # floating ip to nova api gateway
+            # been created, we just return and defer the work to setup bottom
+            # floating ip until vm creation
             return
 
         int_net_pod, b_int_port_id = mappings[0]
