@@ -263,24 +263,14 @@ class TestAsyncJobController(API_FunctionalTest):
         for job_type in self.all_job_types:
             job = self._prepare_job_element(job_type)
 
-            jobs = [
-                {
-                    "job": job,
-                    "expected_error": 200
-                },
-            ]
+            job = {"job": job, "expected_error": 200}
 
-            self._test_and_check(jobs)
+            back_jobid = self._test_and_obtain_id(job)
 
-            response = self.app.get('/v1.0/jobs')
-            return_job = response.json
-
-            all_job_ids[index] = return_job['jobs'][index]['id']
-            all_job_project_ids[job_type] = (
-                return_job['jobs'][index]['project_id'])
+            all_job_ids[index] = back_jobid
+            all_job_project_ids[job_type] = job['job']['project_id']
 
             index = index + 1
-
         service_uris = ['jobs', 'jobs/detail']
         amount_of_all_jobs = len(self.all_job_types)
         # with no filters all jobs are returned
@@ -729,6 +719,15 @@ class TestAsyncJobController(API_FunctionalTest):
                 expect_errors=True)
             self.assertEqual(response.status_int, test_job['expected_error'])
 
+    def _test_and_obtain_id(self, job):
+        response = self.app.post_json(
+            '/v1.0/jobs', dict(job=job['job']),
+            expect_errors=True)
+        self.assertEqual(response.status_int, job['expected_error'])
+
+        back_job = response.json
+        return back_job['job']['id']
+
     def _prepare_job_element(self, job_type):
         # in order to create a job, we need three elements: job type,
         # job resource and project id.
@@ -747,7 +746,8 @@ class TestAsyncJobController(API_FunctionalTest):
         # prepare the project id for job creation, currently job parameter
         # contains job type and job resource information.
         job_type = job['type']
-        if job_type == constants.JT_SEG_RULE_SETUP:
+        if job_type in (constants.JT_SEG_RULE_SETUP,
+                        constants.JT_RESOURCE_RECYCLE):
             project_id = job['resource']['project_id']
         else:
             project_id = uuidutils.generate_uuid()
