@@ -13,13 +13,16 @@
 import copy
 import mock
 from mock import patch
+from oslo_config import cfg
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
+import re
 from six.moves import xrange
-import unittest
+import time
 
 import pecan
 
+from tricircle.api import app
 from tricircle.api.controllers import job
 from tricircle.common import constants
 from tricircle.common import context
@@ -28,6 +31,7 @@ from tricircle.common import xrpcapi
 from tricircle.db import api as db_api
 from tricircle.db import core
 from tricircle.db import models
+from tricircle.tests import base
 
 
 class FakeRPCAPI(xrpcapi.XJobAPI):
@@ -47,8 +51,12 @@ class FakeResponse(object):
         return super(FakeResponse, cls).__new__(cls)
 
 
-class AsyncJobControllerTest(unittest.TestCase):
+class AsyncJobControllerTest(base.TestCase):
     def setUp(self):
+        super(AsyncJobControllerTest, self).setUp()
+
+        cfg.CONF.clear()
+        cfg.CONF.register_opts(app.common_opts)
         core.initialize()
         core.ModelBase.metadata.create_all(core.get_engine())
         self.controller = FakeAsyncJobController()
@@ -177,7 +185,7 @@ class AsyncJobControllerTest(unittest.TestCase):
 
     @patch.object(pecan, 'response', new=FakeResponse)
     @patch.object(context, 'extract_context_from_environ')
-    def test_get_one(self, mock_context):
+    def test_get_one_and_get_all(self, mock_context):
         mock_context.return_value = self.context
 
         # failure case, only admin can list the job's info
@@ -228,6 +236,7 @@ class AsyncJobControllerTest(unittest.TestCase):
                                    resource_id)
             all_job_ids[index] = job_1['id']
             index = index + 1
+            time.sleep(1)
 
             # validate if the id=job_id, get_one(id=job_id) can take effective
             job_2 = self.controller.get_one(job_1['id'])['job']
@@ -244,6 +253,7 @@ class AsyncJobControllerTest(unittest.TestCase):
         for i in xrange(amount_of_succ_jobs):
             db_api.finish_job(self.context, all_job_ids[i], True,
                               timeutils.utcnow())
+            time.sleep(1)
 
         jobs_2 = self.controller.get_one("detail")
         self.assertEqual(amount_of_all_jobs, len(jobs_2['jobs']))
@@ -252,14 +262,24 @@ class AsyncJobControllerTest(unittest.TestCase):
         jobs_3 = self.controller.get_one("detail", **job_status_filter_1)
         self.assertEqual(amount_of_succ_jobs, len(jobs_3['jobs']))
 
+        # set marker in job log
+        res = self.controller.get_all(marker=jobs_3['jobs'][0]['id'],
+                                      limit=amount_of_succ_jobs)
+        self.assertEqual(amount_of_succ_jobs - 1, len(res['jobs']))
+
         job_status_filter_2 = {'status': 'new'}
+        amount_of_new_jobs = amount_of_all_jobs - amount_of_succ_jobs
         jobs_4 = self.controller.get_one("detail", **job_status_filter_2)
-        self.assertEqual(amount_of_all_jobs - amount_of_succ_jobs,
-                         len(jobs_4['jobs']))
+        self.assertEqual(amount_of_new_jobs, len(jobs_4['jobs']))
+
+        # set marker in job
+        res = self.controller.get_all(marker=jobs_4['jobs'][0]['id'],
+                                      limit=amount_of_new_jobs)
+        self.assertEqual(amount_of_new_jobs, len(res['jobs']))
 
     @patch.object(pecan, 'response', new=FakeResponse)
     @patch.object(context, 'extract_context_from_environ')
-    def test_get_all_jobs(self, mock_context):
+    def test_get_all_jobs_with_pagination(self, mock_context):
         mock_context.return_value = self.context
 
         # map job type to project id for later project id filter validation.
@@ -281,10 +301,15 @@ class AsyncJobControllerTest(unittest.TestCase):
                 db_api.register_job(self.context,
                                     job['project_id'], job_type,
                                     resource_id)
+                # because jobs are sorted by timestamp, without time delay then
+                # all jobs are created at the same time, paginate_query can't
+                # identify them
+                time.sleep(1)
             else:
                 db_api.new_job(self.context,
                                job['project_id'], job_type,
                                resource_id)
+                time.sleep(1)
             count = count + 1
 
         # query the jobs with several kinds of filters.
@@ -377,6 +402,91 @@ class AsyncJobControllerTest(unittest.TestCase):
         self.assertEqual(amount_of_running_jobs,
                          len(jobs_job_status_filter_3['jobs']))
 
+        # test for paginate query
+        job_paginate_no_filter_1 = self.controller.get_all()
+        self.assertEqual(amount_of_all_jobs,
+                         len(job_paginate_no_filter_1['jobs']))
+
+        # no limit no marker
+        job_paginate_filter_1 = {'status': 'new'}
+        jobs_paginate_filter_1 = self.controller.get_all(
+            **job_paginate_filter_1)
+        self.assertEqual(amount_of_all_jobs - amount_of_running_jobs,
+                         len(jobs_paginate_filter_1['jobs']))
+
+        # failed cases, unsupported limit type
+        job_paginate_filter_2 = {'limit': '2test'}
+        res = self.controller.get_all(**job_paginate_filter_2)
+        self._validate_error_code(res, 400)
+
+        # successful cases
+        job_paginate_filter_4 = {'status': 'new', 'limit': '2'}
+        res = self.controller.get_all(**job_paginate_filter_4)
+        self.assertEqual(2, len(res['jobs']))
+
+        job_paginate_filter_5 = {'status': 'new', 'limit': 2}
+        res = self.controller.get_all(**job_paginate_filter_5)
+        self.assertEqual(2, len(res['jobs']))
+
+        job_paginate_filter_6 = {'status': 'running', 'limit': 1}
+        res1 = self.controller.get_all(**job_paginate_filter_6)
+
+        marker = res1['jobs'][0]['id']
+        job_paginate_filter_7 = {'status': 'running', 'marker': marker}
+        res2 = self.controller.get_all(**job_paginate_filter_7)
+        self.assertEqual(amount_of_running_jobs, len(res2['jobs']))
+
+        job_paginate_filter_8 = {'status': 'new', 'limit': 3}
+        res = self.controller.get_all(**job_paginate_filter_8)
+        self.assertGreaterEqual(res['jobs'][0]['timestamp'],
+                                res['jobs'][1]['timestamp'])
+        self.assertGreaterEqual(res['jobs'][1]['timestamp'],
+                                res['jobs'][2]['timestamp'])
+
+        # unsupported marker type
+        res = self.controller.get_all(marker=None)
+        self.assertEqual(amount_of_all_jobs, len(res['jobs']))
+
+        res = self.controller.get_all(marker='-123')
+        self._validate_error_code(res, 400)
+
+        # marker not in job table and job log table
+        job_paginate_filter_9 = {'marker': uuidutils.generate_uuid()}
+        res = self.controller.get_all(**job_paginate_filter_9)
+        self._validate_error_code(res, 400)
+
+        # test marker and limit
+        limit = 2
+        pt = '/v1.0/jobs\?limit=\w+&marker=([\w-]+)'
+        job_paginate_filter = {'status': 'new', 'limit': limit}
+        res = self.controller.get_all(**job_paginate_filter)
+        while 'jobs_links' in res:
+            m = re.match(pt, res['jobs_links'][0]['href'])
+            marker = m.group(1)
+            self.assertEqual(limit, len(res['jobs']))
+            job_paginate_filter = {'status': 'new', 'limit': limit,
+                                   'marker': marker}
+            res = self.controller.get_all(**job_paginate_filter)
+
+        job_paginate_filter_10 = {'status': 'running'}
+        res = self.controller.get_all(**job_paginate_filter_10)
+        self.assertEqual(amount_of_running_jobs, len(res['jobs']))
+        # add some rows to job log table
+        for i in xrange(amount_of_running_jobs):
+            db_api.finish_job(self.context, res['jobs'][i]['id'], True,
+                              timeutils.utcnow())
+            time.sleep(1)
+        res_success_log = db_api.list_jobs_from_log(self.context, None)
+        self.assertEqual(amount_of_running_jobs, len(res_success_log))
+
+        res_in_job = db_api.list_jobs(self.context, None)
+        self.assertEqual(amount_of_all_jobs - amount_of_running_jobs,
+                         len(res_in_job))
+
+        job_paginate_filter_11 = {'limit': 2}
+        res = self.controller.get_all(**job_paginate_filter_11)
+        self.assertIsNotNone(res['jobs_links'][0]['href'])
+
     @patch.object(pecan, 'response', new=FakeResponse)
     @patch.object(pecan, 'response', new=mock.Mock)
     @patch.object(context, 'extract_context_from_environ')
@@ -593,7 +703,8 @@ class AsyncJobControllerTest(unittest.TestCase):
         # prepare the project id for job creation, currently job parameter
         # contains job type and job resource information.
         job_type = job['type']
-        if job_type == constants.JT_SEG_RULE_SETUP:
+        if job_type in (constants.JT_SEG_RULE_SETUP,
+                        constants.JT_RESOURCE_RECYCLE):
             project_id = job['resource']['project_id']
         else:
             project_id = uuidutils.generate_uuid()
@@ -613,4 +724,7 @@ class AsyncJobControllerTest(unittest.TestCase):
         self.assertEqual(res[list(res.keys())[0]]['code'], code)
 
     def tearDown(self):
+        cfg.CONF.unregister_opts(app.common_opts)
         core.ModelBase.metadata.drop_all(core.get_engine())
+
+        super(AsyncJobControllerTest, self).tearDown()
